%---------Analysis of 2D SFG plot for the FLUPS setup---------%
%Updated on November the 18th, 2020
%Supplied by Etienne Socie for Matlab with the help of Pr. Niko
%Ernsting and Andres Burgos Caminal
close all
clear all

%File preparation
filename=input('specify file name','s');
fl=importdata(filename);
ntmp02=size(fl,2);
ntmp01=size(fl,1);
TM_raw=fl(1:1,2:ntmp02)';
wl=fl(2:ntmp01,1:1);
sig_raw=fl(2:ntmp01,2:ntmp02);
TM_nb=1:size(sig_raw,2);
TM_nb=TM_nb';

figure(1)
contourf(TM_raw,wl,sig_raw)
xlabel('time / ps')
ylabel('pixel / -')
title('Raw data')

underlineLocation = strfind(filename, '_');
firstPart = filename(underlineLocation+1:length(filename));

%%%%%%%%%%%%%%%%%%%%   CUT OUT THE UNWANTED PART %%%%%%%%%%%%%%%%%%
wl_max=str2double(input('specify pixel final point ','s'));
wl_up=wl(1:wl_max,1:1);
a=size(sig_raw,2);
sig_tot=sig_raw(1:wl_max,1:a);
tmax = find(abs(TM_raw-1) < 0.001);

figure(2)
contourf(TM_nb(1:tmax),wl_up,sig_raw(1:wl_max,1:tmax))
title('Raw pulse contour')
xlabel('time / pix')
ylabel('pixel / -')

%%%%%%%%%% CORRECT THE BACKGROUND %%%%%%%%%%%
%substract by the average of the first 50 pixels
background(:)=mean(sig_tot(1:50,:));
   for i=1:length(TM_raw)
   sig_tot(:,i)=sig_tot(:,i)-background(i);
   end
 
%%%%%%%%%%% Wavelength and energy correction %%%%%%%%%%%%%%%%%%%%%%%%

wl_up=265.35931+0.392372*wl_up; %%%%%%%% CHANGE HERE FOR THE NEW CALIB
wl_corr=1./(1./wl_up-1/1340); 
sig_corr = sig_tot.*wl_up.^2./wl_corr.^2;
figure(3)
contourf(TM_raw(1:tmax),wl_corr,sig_corr(:,1:tmax))
title('Wavelength and energy correction')
xlabel('time / ps')
ylabel('wavelength / nm')

%%%%%%%%%%%%%%%%%% FIT WITH 2D GAUSSIAN %%%%%%%%%%%%%%%%%%%%
%look roughly for the peak
Xcorrpi=str2double(input('Xcorr pixel initial point ','s'));
Xcorrpf=str2double(input('Xcorr pixel final point ','s'));
Xcorrti=str2double(input('Xcorr time initial point ','s'));
Xcorrtf=str2double(input('Xcorr time final point ','s'));

mx=max(max(sig_corr(Xcorrpi:Xcorrpf,Xcorrti:Xcorrtf))); %wl,time
[is,js]=find(sig_corr==mx);

%generate arrays for the fit
Pix=(1:length(wl_up));
[Y,X]=meshgrid(TM_nb,Pix);
ydata=zeros(length(Pix),length(TM_nb),2);
ydata(:,:,1)=X;
ydata(:,:,2)=Y;

%initial guess
aa=0.02;
bb=0.002;
cc=0.02;
x0=[aa,bb,cc,is(1),js(1),mx];

%Least squares fit
x=lsqcurvefit(@D2GaussFunction,x0,ydata,sig_corr);

%Plot results
F0 = D2GaussFunction(x0,ydata);
F = D2GaussFunction(x,ydata);
figure(4)
subplot(1,3,2)
contourf(TM_raw(1:tmax),wl_up,F(:,1:tmax))
title('Gaussian fit')
xlabel('time / ps')
ylabel('pixel / -')
subplot(1,3,3)
contourf(TM_raw(1:tmax),wl_up,F0(:,1:tmax))
title('Gaussian initial guess')
xlabel('time / ps')
ylabel('pixel / -')
subplot(1,3,1)
contourf(TM_raw(1:tmax),wl_up,sig_corr(:,1:tmax))
title('Data')
xlabel('time / ps')
ylabel('pixel / -')

%ANALYZE ROTATION
%it uses the gaussian parameters aa bb and cc
%the following equation must be solved
syms alpha
eqn=0==2*x(2)*cos(alpha)^2+(2*x(3)-2*x(1))*cos(alpha)*sin(alpha)-2*x(2)*sin(alpha)^2;
S=solve(eqn,alpha);

%From the 4 solutions the closest to 0 is used
dS=double(S);
[m,p]=min(abs(dS));

%radians to degrees
angle=real(dS(p)/pi*180);

%INTENSITY AS f(t)
syms ii jj f

%Integrate over pixel
Ifun2=int(exp(1/2*(-ii*(x(1)*ii+x(2)*jj)-jj*(x(2)*ii+x(3)*jj))),ii,[-inf,inf]);
x2=-100:100;
Ifun0=subs(Ifun2,jj,0);
vIfun0=solve(f==Ifun0,f);
Ifun3=simplify(Ifun2/vIfun0);

%find the half maximum position in time
figure(5)
ezplot(Ifun3,[-50,50,-0.2,1.2])
title('int plot')
jj2=(-50:1:50);
intplot4=feval(matlabFunction(Ifun3),jj2);
Ifun4=(Ifun3-0.5)^2;
dIfun4=diff(Ifun4);
dIfun4=simplify(dIfun4);
dIfun4=0==dIfun4;
crit_pts=solve(dIfun4,jj);
k=0;
i=1;
while k==0
    if abs(double(crit_pts(i)))>0
        mpoint=abs(double(crit_pts(i)));
        k=1;
    end
    i=i+1;
end    


%Calculating the FWHM
timeint=0.02; %stepsize around t0
FWHM=2*mpoint*timeint;
di=mpoint;
p_SFG=x(4);
x(6)=1/(1/(x(4)*0.3972+265.35931)-1/1300.7);
t0=(x(5)-1)*timeint+TM_raw(1);

%%%%% Results sheet%%%%%%%
GaussName='F=A*exp(-1/2*(a*(p-cPixPos)^2+2*b*(p-cPixPos)*(t-cTimePos)+c*(t-cTimePos)^2))';
Param_name=strcat(filename,'_Results.txt');
fid=fopen(Param_name,'w');
fprintf(fid,'%s\n','Given the Following 2D Gaussian function:');
fprintf(fid,'%s\n',GaussName);
fprintf(fid,'%s\n','The obtained parameters are:');
fprintf(fid,'%s\t','a');
fprintf(fid,'%s\n',x(1));
fprintf(fid,'%s\t','b');
fprintf(fid,'%s\n',x(2));
fprintf(fid,'%s\t','c');
fprintf(fid,'%s\n',x(3));
fprintf(fid,'%s\t','cPixPos');
fprintf(fid,'%s\n',num2str(x(4)),'(' ,num2str(x(6)),'nm)');
fprintf(fid,'%s\t','cTimePos');
fprintf(fid,'%s\n',strcat(num2str(x(5)),'(' ,num2str(t0),'ps)'));
fprintf(fid,'%s\t','A');
fprintf(fid,'%s\n',x(6));
fprintf(fid,'\n');
fprintf(fid,'%s\t','Angle:');
fprintf(fid,'%s\n',num2str(angle));
fprintf(fid,'%s\t','FWHM (ps):');
fprintf(fid,'%s\n',num2str(FWHM));
t0_name=strcat(filename,'_t0.txt');
fid=fopen(t0_name,'w');
fprintf(fid,'%s',strcat(num2str(t0)));

%%%%%%%%%%%%%%%%%%%%%%%%% Substract real time 0 %%%%%%%%%%%%
TM_real=TM_raw-t0;
figure(6)
contourf(TM_real(1:tmax),wl_up,sig_corr(1:wl_max,1:tmax))
title('Pulse t0 corrected')
xlabel('time / ps')
ylabel('pixel / -')
