%---------Analysis of 2D SFG plot for the FLUPS setup---------%
%Updated on November the 18th, 2020
%Supplied by Etienne Socie

close all
clearvars -except wl_max

%File preparation
filename=input('specify file name','s');
t0=strcat(filename,'_t0.txt');
t0=importdata(t0);
fl=importdata(filename);
ntmp02=size(fl,2);
ntmp01=size(fl,1);
TM_raw=fl(1:1,2:ntmp02)';
wl=fl(2:ntmp01,1:1);
sig_raw=fl(2:ntmp01,2:ntmp02);

figure(1)
contourf(TM_raw,wl,sig_raw)
xlabel('time / ps')
ylabel('wavelength / nm')
title('Raw data')

%Prepare the files name
underlineLocation = strfind(filename, '_');
firstPart = filename(underlineLocation+1:length(filename));

%%%%%%%%%% CORRECT THE BACKGROUND %%%%%%%%%%%
%substract by the average of the first 50 pixels
background(:)=mean(sig_raw(1:50,:));
   for i=1:length(TM_raw)
   sig_raw_bg(:,i)=sig_raw(:,i)-background(i);
   end

%Normalize the signal to 1s of aquisition
expt=str2double(input('specify exposure time ','s'));
sig_raw_bg_norm=sig_raw_bg./expt;

%%%%%%%%%%% Wavelength and energy correction %%%%%%%%%%%%%%%%%%%%%%%%
%Here we have to multiply the signal by lambda(up)^2/lambda(fluo)^2.
wl_up=wl(1:wl_max,1:1);
wl_up=265.35931+0.392372*wl_up;
wl_corr=1./(1./wl_up-1/1340); 
a=size(sig_raw_bg_norm,2);
sig_corr = sig_raw_bg_norm(1:wl_max,1:a).*wl_up.^2./wl_corr.^2;
k = find(abs(TM_raw-20) < 0.01);
figure(3)
contourf(TM_raw(1:k,:),wl_corr,sig_corr(1:wl_max,1:k))
title('Wavelength and energy correction')
xlabel('time / ps')
ylabel('wavelength / nm')

%%%%%%%%%%%%%%%% Chirp correction %%%%%%%%%%%%%%%%%%%%

%Create theoritical GDD with the path length in glass 
% https://www.newport.com/n/the-effect-of-dispersion-on-ultrashort-pulses
GVD=[0.09757,0.08255,0.07150,0.06291,0.05593,0.05005,0.04493,0.04036,0.03616,0.03223];
GDD=2.7*GVD; 
GDD=GDD-GDD(1);
GDD=GDD';
GDD_wl=[402,450,500,550,600,650,700,750,800,850]';
f = @(b,GDD_wl) b(1)+b(2).*exp(-b(3).*GDD_wl);                                   
B = fminsearch(@(b) norm(GDD - f(b,GDD_wl)),[-0.2;1; 0.004]);


%create shifted timedelays
TM_real=TM_raw-t0;
td_x(1:length(wl_corr),length(TM_real))=0;
for i=1:length(wl_corr)
    zero=B(1)+B(2).*exp(-wl_corr(i).*B(3)); 
    for j=1:length(TM_real)
    td_x(i,j) = TM_real(j)-zero;
    end
end

%interpolate for chirp correction
Data_chirp(1:length(wl_corr),length(TM_real))=0;
for i=1:length(wl_corr)
Data_chirp(i,:)=interp1(td_x(i,:),sig_corr(i,:),TM_real,'pchip');
end

figure(4)
contourf(TM_real(1:k,:),wl_corr,Data_chirp(:,1:k))
title('Pulse chirp corrected')
xlabel('time / ps')
ylabel('wavelength / nm')


%%%%%%%%%%%% Plotting the signal %%%%%%%%%%%%%%%%
wl_i=str2double(input('Select starting wavelength (upconverted) ','s'));
wl_e=str2double(input('Select final wavelength (upconverted) ','s'));
wl_i=round((wl_i-265.35931)/0.3972); %CHANGE IT FOR THE NEW CALIB
wl_e=round((wl_e-265.35931)/0.3972);
mxp=max(max(sig_corr(200:240,40:100)));%Data_chirp(175:190,90:140))); %wl,time

figure(5)
contourf(TM_real(1:236,:),wl_corr(wl_i:wl_e,1:1),sig_corr(wl_i:wl_e,1:236),[-mxp/3,mxp/7.5,mxp/3.75,mxp/2.5,mxp/1.87,mxp/1.5])
xlabel('Time / ps')

ylabel('Wavelength / nm')
c = colorbar;
c.Label.String = 'Upconverted PL / a.u';
c.Label.FontSize = 24;
c.LineWidth=1;
c.Box='off';
set(gca,'FontName','Helvetica','FontSize',24,'TickDir','both','TickLength',[0.01,0.01],'LineWidth',2.5)

Save=str2double(input('Do you want to save .eps image? yes:1 no:2','s'));
if Save==1
saveas(figure(5),firstPart,'epsc');
end

%%%%%%%%%% Exporting everyything %%%%%%%%%%%%
TM_round=2*floor(round(100*round(TM_real,2))/2)/100;
wl_round=round(wl_corr,1);

%Export spectral slices
Spectra_label=[];
for i=1:length(TM_real)
    Spectra_label=strvcat(Spectra_label, [firstPart,num2str(TM_round(i)),'ps']);
end
Spectra_label=cellstr(Spectra_label);
dyn_name=strcat(firstPart,'_spec.txt');
fid=fopen(dyn_name,'w');
for i = 1:length(TM_real)
    LabelString = cell2mat(Spectra_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for i=1:length(wl_corr)
    for j=1:length(TM_real)
        fprintf(fid,'%e\t',sig_corr(i,j));
    end
    fprintf(fid,'\n');
end
fclose(fid);


TM_label=[];
TM_label=strvcat(TM_label, ['time',firstPart]);
TM_label=cellstr(TM_label);
TM_name_MA=strcat(firstPart,'_time.txt');
fid=fopen(TM_name_MA,'w');
LabelString = cell2mat(TM_label);
fprintf(fid, '%s', LabelString);
fprintf(fid, '\n');
for i=1:length(TM_round)
    fprintf(fid,'%.2f\n',TM_real(i));
end

wl_label=[];
wl_label=strvcat(wl_label, ['wl',firstPart]);
wl_label=cellstr(wl_label);
wl_name=strcat(firstPart,'_wl.txt');
fid=fopen(wl_name,'w');
LabelString = cell2mat(wl_label);
fprintf(fid, '%s', LabelString);
fprintf(fid, '\n');
for i=1:length(wl_corr)
    fprintf(fid,'%.1f\n',wl_corr(i));
end  



Dynamics_label=[];
for i=1:length(wl_corr)
    Dynamics_label=strvcat(Dynamics_label, [firstPart,num2str(wl_round(i)),'nm']);
end
Dynamics_label=cellstr(Dynamics_label);
spec_name=strcat(firstPart,'_dyn.txt');
fid=fopen(spec_name,'w');
spec=(sig_corr)';
for i = 1:length(wl_corr)
    LabelString = cell2mat(Dynamics_label(i)); 
    fprintf(fid, '%s\t', LabelString);
end
fprintf(fid, '\n');
for i=1:length(TM_real)
    for j=1:length(wl_corr)
        fprintf(fid,'%e\t',spec(i,j));
    end
    fprintf(fid,'\n');
end
fclose(fid);